"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.promiseMiddleware = exports.createAction = exports.REJECTED = exports.FULFILLED = exports.PENDING = exports.isValid = exports.isPromise = void 0;

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var isPromise = function isPromise(payload) {
  return Promise.resolve(payload) === payload;
};

exports.isPromise = isPromise;

var isValid = function isValid(action) {
  return _typeof(action) === 'object' && action.hasOwnProperty('type') && isPromise(action.payload);
};

exports.isValid = isValid;
var PENDING = 'PENDING';
exports.PENDING = PENDING;
var FULFILLED = 'FULFILLED';
exports.FULFILLED = FULFILLED;
var REJECTED = 'REJECTED';
exports.REJECTED = REJECTED;

var createAction = function createAction(type, promiseFn) {
  if (typeof promiseFn !== 'function') {
    throw "The second argument to createAction must be a function. Received '".concat(_typeof(promiseFn), "'");
  }

  var pendingType = "".concat(type, "_").concat(PENDING);
  var fulfilledType = "".concat(type, "_").concat(FULFILLED);
  var rejectedType = "".concat(type, "_").concat(REJECTED);
  return [function () {
    return {
      type: type,
      payload: promiseFn.apply(void 0, arguments),
      pendingType: pendingType,
      fulfilledType: fulfilledType,
      rejectedType: rejectedType
    };
  }, type, pendingType, fulfilledType, rejectedType];
};

exports.createAction = createAction;

var promiseMiddleware = function promiseMiddleware(_ref) {
  var dispatch = _ref.dispatch;
  return function (next) {
    return function (action) {
      if (!isValid(action)) {
        return next(action);
      }

      var type = action.type,
          _action$pendingType = action.pendingType,
          pendingType = _action$pendingType === void 0 ? PENDING : _action$pendingType,
          _action$fulfilledType = action.fulfilledType,
          fulfilledType = _action$fulfilledType === void 0 ? FULFILLED : _action$fulfilledType,
          _action$rejectedType = action.rejectedType,
          rejectedType = _action$rejectedType === void 0 ? REJECTED : _action$rejectedType,
          payload = action.payload,
          rest = _objectWithoutProperties(action, ["type", "pendingType", "fulfilledType", "rejectedType", "payload"]);

      dispatch({
        type: pendingType
      });
      payload.then(function (payload) {
        dispatch(_objectSpread({
          type: type,
          payload: payload
        }, rest));
        dispatch({
          type: fulfilledType
        });
      }).catch(function (err) {
        dispatch({
          type: rejectedType,
          payload: err
        });
      });
    };
  };
};

exports.promiseMiddleware = promiseMiddleware;
//# sourceMappingURL=index.js.map